/** 
 * Rebar - Reading Progress Bar for WordPress, exclusively on Envato Market: https://1.envato.market/rebar-wp
 * @encoding     UTF-8
 * @version      1.0.2
 * @copyright    Copyright (C) 2019 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license      Envato Standard License https://1.envato.market/KYbje
 * @author       Alexandr Khmelnytsky ( info@alexander.khmelnitskiy.ua )
 * @support      dmitry@merkulov.design
 **/

/*
 * scrollProgress - v2.1.2
 * https://github.com/jeremenichelli/scrollProgress
 * 2014 (c) Jeremias Menichelli - MIT License
*/

(function(root, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(function() {
            return factory(root);
        });
    } else if (typeof exports === 'object') {
        module.exports = factory;
    } else {
        root.scrollProgress = factory(root);
    }
})(this, function() {
    'use strict';

    var body = document.body,
        progress = 0,
        isSet = false,
        progressWrapper,
        progressElement,
        endPoint,
        // default configuration object
        config = {
            bottom: true,
            color: '#000000',
            height: '5px',
            styles: true,
            prefix: 'progress',
            events: true
        };

    /*
     * Create DOM elements which graphically represent the progress
     * @method _createElements
     */
    var _createElements = function() {
        progressWrapper = document.createElement('div');
        progressElement = document.createElement('div');

        progressWrapper.id = config.prefix + '-wrapper';
        if(window.md_rebar_style){
            progressWrapper.className = window.md_rebar_style;
        }
        progressElement.id = config.prefix + '-element';

        progressWrapper.appendChild(progressElement);
        document.body.appendChild(progressWrapper);
        
    };

    /*
     * Replaces configuration values with custom ones
     * @method _setConfigObject
     * @param {object} obj - object containing custom options
     */
    var _setConfigObject = function(obj) {
        // override with custom attributes
        if (typeof obj === 'object') {
            for (var key in config) {
                if (typeof obj[key] !== 'undefined') {
                    config[key] = obj[key];
                }
            }
        }
    };

    /*
     * Set styles on DOM elements
     * @method _setElementsStyles
     */
    var _setElementsStyles = function() {
        // setting progress to zero and wrapper to full width
        progressElement.style.width = '0';
        progressWrapper.style.width = '100%';

        // set styles only if
        // settings is true
        if (config.styles) {
            // progress element
            progressElement.style.backgroundColor = config.color;
            progressElement.style.boxShadow = '1px 1px 5px ' + config.color;
            progressElement.style.height = config.height;

            // progress wrapper
            progressWrapper.style.position = 'fixed';
            progressWrapper.style.left = '0';

            // sets position
            if (config.bottom) {
                progressWrapper.style.bottom = '0';
            } else {
                progressWrapper.style.top = '0';
            }
        }
    };

    /*
     * Main function which sets all variables and bind events if needed
     * @method _set
     * @param {object} custom - object containing custom options
     */
    var _set = function(custom) {
        // set only once
        if (!isSet) {
            if (custom) {
                _setConfigObject(custom);
            }
            _createElements();
            _setElementsStyles();

            // set initial metrics
            _setMetrics();

            // bind events only if
            // settings is true
            if (config.events) {
                window.onscroll = _setProgress;
                window.onresize = _setMetrics;
            }

            isSet = true;
        } else {
            throw new Error('scrollProgress has already been set!');
        }
    };

    /*
     * Calculates how much user has scrolled
     * @method _setProgress
     */
    var _setProgress = function() {
        try {
            var y = window.scrollY || window.pageYOffset || document.documentElement.scrollTop;
            progress = y / endPoint * 100;
            progressElement.style.width = progress + '%';
        } catch (e) {
            console.error(e);
        }
    };

    /*
     * Updates the document's height and adjusts the progress bar
     * @method _setMetrics
     */
    var _setMetrics = function() {
        endPoint = _getEndPoint();
        _setProgress();
    };

    /*
     * Returns how much the user can scroll in the document
     * @method _getEndPoint
     */
    var _getEndPoint = function() {
        return document.body.scrollHeight - (window.innerHeight || document.documentElement.clientHeight);
    };

    return {
        set: _set,
        trigger: _setProgress,
        update: _setMetrics
    };
});

/** Initialize Reading Progress Bar */
window.onload = function () {

    scrollProgress.set({
        prefix: 'md-rebar',
        color: window.md_rebar_color,
        height: window.md_rebar_height + 'px',
        styles: true,
        bottom: (window.md_rebar_bottom == "1" ? true : false) 
    });
    window.onresize = scrollProgress.update;
    window.onscroll = scrollProgress.trigger;
};